@media screen{@import url(sheet2.html);foo{asdf:asdf;ew:12;}bar{content:"a\n  b\
    asdf";}}.clearfix:after{content:".";display:block;height:0;clear:both;visibility:hidden;}.clearfix{display:inline-block;}* html .clearfix{height:1%;}.clearfix{display:block;}